@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001, 2002, 2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -42,7 +42,7 @@ typedef struct {
 		struct in_addr  ip4;
 #ifdef HAVE_IPV6
 		struct in6_addr ip6;
-#endif	
+#endif
 	};
 } ip_t;
 
@@ -103,16 +103,16 @@ print_ip (ip_type_t type, ip_t *ip)
 #ifdef HAVE_INET_PTON
 # ifdef HAVE_IPV6
 	if (type == ipv6) {
-		printf ("6(%s)", inet_ntop (AF_INET6, ip, dir, dir_size));
+		printf (inet_ntop (AF_INET6, ip, dir, dir_size));
 		return;
 	}
 # endif
 	if (type == ipv4) {
-		printf ("4(%s)", inet_ntop (AF_INET, ip, dir, dir_size));
+		printf (inet_ntop (AF_INET, ip, dir, dir_size));
 		return;
 	}
 #else
-	printf ("4(%s)", inet_ntoa (ip.ipv4));
+	printf ("%s", inet_ntoa (ip.ipv4));
 #endif
 }
 
@@ -155,13 +155,25 @@ parse_ip (char *ip, ip_item_t *n)
 	 */
 #ifdef HAVE_INET_PTON
 	ok = (inet_pton (n->type, ip, &n->ip) > 0);
+
+#ifdef HAVE_IPV6
+	if (n->type == ipv6) {
+		if (IN6_IS_ADDR_V4MAPPED (&(n->ip).ip6)) {
+			PRINT_ERROR ("This IP '%s' is IPv6-mapped IPv6 address.  "
+				     "Please, specify IPv4 in a.b.c.d style instead "
+				     "of ::ffff:a.b.c.d style\n", ip);
+			return ret_error;
+		}
+	}
+#endif /* HAVE_IPV6 */
+
 #else
 	ok = (inet_aton (ip, &n->ip) != 0);
 #endif
 
 	return (ok) ? ret_ok : ret_error;
 }
-
+ 
 
 static ret_t
 parse_netmask (char *netmask, subnet_item_t *n)
@@ -188,7 +200,7 @@ parse_netmask (char *netmask, subnet_item_t *n)
 	/* Lenght mask
 	 * Eg: 16
 	 */
-	if (strlen(netmask) > 2) {
+	if (strlen(netmask) > 3) {
 		return ret_error;
 	}
 
@@ -196,9 +208,12 @@ parse_netmask (char *netmask, subnet_item_t *n)
 	if (num <= LONG_MIN) 
 		return ret_error;
 
-	/* Sanity check 
+	/* Sanity checks
 	 */
-	if (num > (IP_NODE(n)->type == ipv4) ? 32 : 128) 
+	if ((IP_NODE(n)->type == ipv4) && (num >  32)) 
+		return ret_error;
+
+	if ((IP_NODE(n)->type == ipv6) && (num > 128)) 
 		return ret_error;
 
 #ifdef HAVE_IPV6
@@ -206,48 +221,39 @@ parse_netmask (char *netmask, subnet_item_t *n)
 		return ret_error;
 	}
 
-	if (IP_NODE(n)->type == ipv6) {
-		int i;
+	/* Special case
+	 */
+	if (num == 128) {
+		n->mask.ip6.in6_u.u6_addr32[0] = ~0L;
+		n->mask.ip6.in6_u.u6_addr32[1] = ~0L;
+		n->mask.ip6.in6_u.u6_addr32[2] = ~0L;
+		n->mask.ip6.in6_u.u6_addr32[3] = ~0L;
 
-		memset (&n->mask, 1, sizeof(struct in6_addr));
-
-		if (num <= 32) {
-			n->mask.ip6.in6_u.u6_addr32[3] = (~0 << (32-num));
-		}
-
-		if (num <= 64) {
-			n->mask.ip6.in6_u.u6_addr32[2] = (~0 << (64-num));		
-		}
-
-
-
-		if (num > 96) {
-			n->mask.ip6.in6_u.u6_addr32[0] = (~0 << (128-num));
-			num -= 32;
-		}
-
-		if (num > 64) {
-			n->mask.ip6.in6_u.u6_addr32[1] = (~0 << (96-num));
-			num -= 32;
-		}
-
-		if (num > 32) {
-			n->mask.ip6.in6_u.u6_addr32[2] = (~0 << (64-num));
-			num -= 32;
-		}
-
-	} else
-#endif
-	{
-		memset (&n->mask, 0, sizeof(struct in_addr));
+		return ret_ok;
 	}
 
+	if (IP_NODE(n)->type == ipv6) {
+		int i, j, jj;
+		unsigned char mask    = 0;
+		unsigned char maskbit = 0x80L;
 
-	// TODO: Write support for this
-	PRINT_ERROR ("TODO: support netmasks like this one '%s'. %s,%d\n", 
-		     netmask, __FILE__, __LINE__);
-	return ret_error;
-	//
+		j  = (int) num / 8;
+		jj = num % 8;
+
+		for (i=0; i<j; i++) {
+			n->mask.ip6.in6_u.u6_addr8[i] = 0xFF;
+		}
+
+		while (jj--) {
+			mask |= maskbit;
+			maskbit >>= 1;
+		}
+		n->mask.ip6.in6_u.u6_addr8[j] = mask;
+
+		return ret_ok;
+	} else
+#endif
+		n->mask.ip4.s_addr = (in_addr_t) htonl(~0L << (32 - num));
 
 	return ret_ok;
 }
@@ -256,6 +262,7 @@ parse_netmask (char *netmask, subnet_item_t *n)
 ret_t 
 cherokee_access_add_ip (cherokee_access_t *entry, char *ip)
 {
+	ret_t ret;
 	ip_item_t *n;
 
 	n = new_ip();
@@ -263,7 +270,12 @@ cherokee_access_add_ip (cherokee_access_t *entry, char *ip)
 
 	list_add ((list_t *)n, &entry->list_ips);
 
-	return parse_ip (ip, n);
+	ret = parse_ip (ip, n);
+	if (ret < ret_ok) {
+		PRINT_ERROR ("IP address '%s' seems to be invalid\n", ip);
+	}
+
+	return ret;
 }
 
 
@@ -271,20 +283,18 @@ ret_t
 cherokee_access_add_subnet (cherokee_access_t *entry, char *subnet)
 {
 	char *slash;
-	char *ip; 
 	char *mask;
 	ret_t ret;
 	subnet_item_t *n;
+	CHEROKEE_NEW (ip,buffer);
 
 	/* Split the string
 	 */
 	slash = strpbrk (subnet, "/\\");
 	if (slash == NULL) return ret_error;
 
-	ip   = subnet;
 	mask = slash +1;
-
-	slash[0] = '\0';
+	cherokee_buffer_add (ip, subnet, mask-subnet-1);
 	
 	/* Create the new list object
 	 */
@@ -295,19 +305,25 @@ cherokee_access_add_subnet (cherokee_access_t *entry, char *subnet)
 
 	/* Parse the IP
 	 */
-	ret = parse_ip (ip, IP_NODE(n));
-	if (ret < ret_ok) goto error;
+	ret = parse_ip (ip->buf, IP_NODE(n));
+	if (ret < ret_ok) {
+		PRINT_ERROR ("IP address '%s' seems to be invalid\n", ip->buf);
+		goto error;
+	}
 
 	/* Parse the Netmask
 	 */
 	ret = parse_netmask (mask, n);
-	if (ret < ret_ok) goto error;	
+	if (ret < ret_ok) {
+		PRINT_ERROR ("Netmask '%s' seems to be invalid\n", mask);
+		goto error;	
+	}
 
-	*slash = '/';
+	cherokee_buffer_free (ip);
 	return ret_ok;
 
 error:
-	*slash = '/';
+	cherokee_buffer_free (ip);
 	return ret_error;
 }
 
@@ -368,7 +384,62 @@ cherokee_access_ip_match (cherokee_access_t *entry, cherokee_socket_t *sock)
 	
 	/* Check in the Subnets list
 	 */
+	list_for_each (i, (list_t*)&entry->list_subnets) {
+		int j, re;
+		ip_t masqued_remote, masqued_list;
+		
+		if (SOCKET_AF(sock) == IP_NODE(i)->type) {
+			switch (IP_NODE(i)->type) {
+			case ipv4:
+				printf ("Es IPv4\n");
 
+				masqued_list.ip4.s_addr   = (IP_NODE(i)->ip.ip4.s_addr &
+							     SUBNET_NODE(i)->mask.ip4.s_addr);
+				masqued_remote.ip4.s_addr = (SOCKET_ADDR_IPv4(sock)->sin_addr.s_addr & 
+							     SUBNET_NODE(i)->mask.ip4.s_addr);
+
+				if (masqued_remote.ip4.s_addr == masqued_list.ip4.s_addr) {
+					return ret_ok;
+				}
+
+				break;
+#ifdef HAVE_IPV6
+			case ipv6:
+			{
+				int equal = 1;
+
+//				printf ("remote IPv6: ");
+//				print_ip (ipv6, &SOCKET_ADDR_IPv6(sock)->sin6_addr);
+//				printf ("\n");
+
+				for (j=0; j<16; j++) {
+					masqued_list.ip6.in6_u.u6_addr8[j] = (
+						IP_NODE(i)->ip.ip6.in6_u.u6_addr8[j] &
+						SUBNET_NODE(i)->mask.ip6.in6_u.u6_addr8[j]);
+					masqued_remote.ip6.in6_u.u6_addr8[j] = (
+						SOCKET_ADDR_IPv6(sock)->sin6_addr.in6_u.u6_addr8[j] &
+						SUBNET_NODE(i)->mask.ip6.in6_u.u6_addr8[j]);
+
+					if (masqued_list.ip6.in6_u.u6_addr8[j] !=
+					    masqued_remote.ip6.in6_u.u6_addr8[j])
+					{
+						equal = 0;
+						break;
+					}
+				}
+
+				if (equal == 1) {
+					return ret_ok;
+				}
+				break;
+			}
+#endif
+			default:
+				SHOULDNT_HAPPEN;
+				return ret_error;
+			}
+		}
+	}
 	
 	return ret_not_found;
 }
