@@ -6,7 +6,7 @@
  *      Ayose Cazorla Len <setepo@gulic.org>
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001, 2002, 2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -91,15 +91,17 @@ cherokee_handler_cgi_new  (cherokee_handler_t **hdl, void *cnt, cherokee_table_t
 	
 	/* Init
 	 */
-	n->pipeInput = 0;
-	n->pipeOutput = 0;
+	n->pipeInput      = 0;
+	n->pipeOutput     = 0;
 	n->post_data_sent = 0;
-	n->pid = 0;
-	n->pathInfo = NULL;
-	n->fileName = NULL;
-	n->data = NULL;
-	n->parameter = NULL;
+	n->pid            = 0;
+	n->pathInfo       = NULL;
+	n->fileName       = NULL;
+	n->data           = NULL;
+	n->parameter      = NULL;
 
+	/* Return the object
+	 */
 	*hdl = HANDLER(n);
 
 	return ret_ok;
@@ -135,35 +137,35 @@ _set_envs_cgi (cherokee_connection_t *conn, cherokee_handler_cgi_t* n)
 	r += setenv ("HTTP_HOST", tmp->buf, 1);
 
 	if ((p = index(tmp->buf, ':')) != NULL) *p = '\0';
-	r += setenv("SERVER_NAME", tmp->buf, 1);
+	r += setenv ("SERVER_NAME", tmp->buf, 1);
 
 
 	/* Cookies :-)
 	 */
 	cherokee_buffer_clean(tmp);
-	ret = cherokee_header_copy_unknown(conn->header, "Cookie", 6, tmp);
+	ret = cherokee_header_copy_unknown (conn->header, "Cookie", 6, tmp);
 	if (ret == ret_ok)
-		r += setenv("HTTP_COOKIE", tmp->buf, 1);
+		r += setenv ("HTTP_COOKIE", tmp->buf, 1);
 
 	/* User Agent
 	 */
 	cherokee_buffer_clean (tmp);
 	ret = cherokee_header_copy_known (conn->header, header_user_agent, tmp);
 	if (ret == ret_ok) {
-		r += setenv("HTTP_USER_AGENT", tmp->buf, 1);
+		r += setenv ("HTTP_USER_AGENT", tmp->buf, 1);
 	}
 
 	/* Content-type and Content-lenght (if available) 
 	 */
 	cherokee_buffer_clean(tmp);
-	ret = cherokee_header_copy_unknown(conn->header, "Content-Type", 12, tmp);
+	ret = cherokee_header_copy_unknown (conn->header, "Content-Type", 12, tmp);
 	if (ret == ret_ok)
-		r += setenv("CONTENT_TYPE", tmp->buf, 1);
+		r += setenv ("CONTENT_TYPE", tmp->buf, 1);
 
 	cherokee_buffer_clean(tmp); 
-	ret = cherokee_header_copy_known(conn->header, header_content_length, tmp);
+	ret = cherokee_header_copy_known (conn->header, header_content_length, tmp);
 	if (ret == ret_ok)
-		r += setenv("CONTENT_LENGTH", tmp->buf, 1);
+		r += setenv ("CONTENT_LENGTH", tmp->buf, 1);
 
 	cherokee_buffer_free(tmp);
 
@@ -171,19 +173,19 @@ _set_envs_cgi (cherokee_connection_t *conn, cherokee_handler_cgi_t* n)
 	 * doens't have the \0 in the correct place (at start)
 	 */
 	if(conn->query_string->len > 0)
-		r += setenv("QUERY_STRING", conn->query_string->buf, 1);
+		r += setenv ("QUERY_STRING", conn->query_string->buf, 1);
 	else
-		r += setenv("QUERY_STRING", "", 1);
+		r += setenv ("QUERY_STRING", "", 1);
 
 	snprintf (temp, temp_size, "%d", CONN_SRV(conn)->port);
-	r += setenv("SERVER_PORT", temp, 1);
+	r += setenv ("SERVER_PORT", temp, 1);
 
 	switch (conn->header->version) {
 	case http_version_11:
-		r += setenv("SERVER_PROTOCOL", "HTTP/1.1", 1);
+		r += setenv ("SERVER_PROTOCOL", "HTTP/1.1", 1);
 		break;
 	case http_version_10:
-		r += setenv("SERVER_PROTOCOL", "HTTP/1.0", 1);
+		r += setenv ("SERVER_PROTOCOL", "HTTP/1.0", 1);
 		break;
 	default:
 		SHOULDNT_HAPPEN;
@@ -191,13 +193,13 @@ _set_envs_cgi (cherokee_connection_t *conn, cherokee_handler_cgi_t* n)
 
 	switch (conn->header->method) {
 	case http_get:
-		r += setenv("REQUEST_METHOD", "GET", 1);		   
+		r += setenv ("REQUEST_METHOD", "GET", 1);		   
 		break;
 	case http_post:
-		r += setenv("REQUEST_METHOD", "POST", 1);		   
+		r += setenv ("REQUEST_METHOD", "POST", 1);		   
 		break;
 	case http_head:
-		r += setenv("REQUEST_METHOD", "HEAD", 1);		   
+		r += setenv ("REQUEST_METHOD", "HEAD", 1);		   
 		break;
 	default:
 		SHOULDNT_HAPPEN;
@@ -208,7 +210,7 @@ _set_envs_cgi (cherokee_connection_t *conn, cherokee_handler_cgi_t* n)
 	}
 
 	if (conn->header->method < 4) {
-		r = setenv("REQUEST_METHOD", method[conn->header->method], 1);
+		r = setenv ("REQUEST_METHOD", method[conn->header->method], 1);
 		if (r != 0) return ret_error;
 	}
 
@@ -407,12 +409,17 @@ cherokee_handler_cgi_init (cherokee_handler_cgi_t *n)
 		_LOG("CGI: Executing %s %s\n", n->fileName->buf,
 		     (n->parameter)? n->parameter->buf : "");
 
-		/* Execute the new CGI 
+		/* Change the current directory to to the program's own directory.
 		 */
 		absolute_path = n->fileName->buf;
-		file = strrchr(absolute_path, '/') + 1;
-
+		file = strrchr (absolute_path, '/');
+		*file = '\0';
+		chdir (absolute_path);
+		*file = '/';
+		file++;
 
+		/* Execute the new CGI 
+		 */
 		if (n->parameter == NULL) {
 			execl (absolute_path, file, NULL);
 		} else {
@@ -630,7 +637,7 @@ static void
 child_finished(int sng)
 {
 	int status;
-	waitpid (0, &status, WNOHANG);
+	while(waitpid (0, &status, WNOHANG) > 0);
 }
 
 void
