@@ -348,19 +348,25 @@ initialize_server_socket6 (cherokee_server_t *srv, unsigned short port, int *srv
 static ret_t
 print_banner (cherokee_server_t *srv)
 {
-	CHEROKEE_NEW(n,buffer);
-
-	cherokee_buffer_add_va (n, "Cherokee Web Server %s: ", VERSION);
-	cherokee_buffer_add_va (n, "Listening on port %d", srv->port);
-	if (srv->thread_num < 1) {
-		cherokee_buffer_add (n, ", single thread", 15);
-	} else {
- 		cherokee_buffer_add_va (n, ", %d threads", srv->thread_num+1);
-	}
+	   char *method;
+	   CHEROKEE_NEW(n,buffer);
 	
-	printf ("%s\n", n->buf);
 
-	cherokee_buffer_free (n);
+	   cherokee_buffer_add_va (n, "Cherokee Web Server %s: ", VERSION);
+	   cherokee_buffer_add_va (n, "Listening on port %d", srv->port);
+	   if (srv->thread_num < 1) {
+			 cherokee_buffer_add (n, ", single thread", 15);
+	   } else {
+			 cherokee_buffer_add_va (n, ", %d threads", srv->thread_num+1);
+	   }
+	   
+	   cherokee_fdpoll_get_method (srv->main_thread->fdpoll, &method);
+	   cherokee_buffer_add_va (n, ", using %s", method);
+						  
+	   printf ("%s\n", n->buf);
+	   cherokee_buffer_free (n);
+
+	   return ret_ok;
 }
 
 
