@@ -8,7 +8,7 @@
  * This piece of code by:
  *      Ricardo Cardenes Medina <ricardo@conysis.com>
  *
- * Copyright (C) 2001, 2002, 2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -272,15 +272,15 @@ cherokee_socket_ntop (cherokee_socket_t *socket, char *dst, size_t cnt)
 # ifdef HAVE_IPV6
 	if (SOCKET_AF(socket) == AF_INET6) {
 		/* We assume that inet_ntop is there too */
-		ret = inet_ntop(AF_INET6,
-				&((struct sockaddr_in6 *)&SOCKET_ADDR(socket))->sin6_addr,
-				dst, cnt);
+		ret = (char *) inet_ntop(AF_INET6,
+					 &((struct sockaddr_in6 *)&SOCKET_ADDR(socket))->sin6_addr,
+					 dst, cnt);
 	}
 # endif /* HAVE_IPV6 */
 	if (SOCKET_AF(socket) == AF_INET) {
-		ret = inet_ntop(AF_INET,
-				&((struct sockaddr_in *)&SOCKET_ADDR(socket))->sin_addr,
-				dst, cnt);
+		ret = (char *) inet_ntop(AF_INET,
+					 (void *) &((struct sockaddr_in *)&SOCKET_ADDR(socket))->sin_addr,
+					 dst, cnt);
 	}
 #else /* !HAVE_INET_PTON */
 	if (SOCKET_AF(socket) == AF_INET) {
@@ -295,7 +295,7 @@ cherokee_socket_ntop (cherokee_socket_t *socket, char *dst, size_t cnt)
 
 
 ret_t
-cherokee_socket_accept (cherokee_socket_t  *socket, int server_socket)
+cherokee_socket_accept (cherokee_socket_t *socket, int server_socket)
 {
 	int tmp = 1;
 	int new_socket;
@@ -307,6 +307,7 @@ cherokee_socket_accept (cherokee_socket_t  *socket, int server_socket)
 		return ret_error;
 	}
 
+
 	/* Set some parameters to the socket
 	 */
 
@@ -347,13 +348,13 @@ cherokee_socket_set_status (cherokee_socket_t  *socket, cherokee_socket_status_t
 
 
 static ret_t 
-cherokee_write (cherokee_socket_t *socket, const char *buf, int len)
+cherokee_write (cherokee_socket_t *socket, const char *buf, int buf_len)
 {
-	int rc;
+	ssize_t rc;
 
 #if HAVE_TLS
 #else
-	rc = write (socket, buf, len);
+	rc = write (SOCKET_FD(socket), buf, buf_len);
 #endif
 
 	if (rc < 0) {
@@ -365,20 +366,20 @@ cherokee_write (cherokee_socket_t *socket, const char *buf, int len)
 
 
 static ret_t
-cherokee_read (cherokee_socket_t *socket, char *buf, int size, int *readed)
+cherokee_read (cherokee_socket_t *socket, char *buf, int buf_size, int *readed)
 {
-	int rc;
+	ssize_t rc;
 
 #ifdef HAVE_TLS
 #else	
-	rc = read (SOCKET_FD(socket), buf, size);
+	rc = read (SOCKET_FD(socket), buf, buf_size);
 #endif
 
 	if (rc < 0) {
 		return ret_error;
 	}
 	
-	*readed = rc;
+	*readed = (int)rc;
 	return ret_ok;
 }
 
