@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001, 2002, 2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -51,6 +51,11 @@ set_sigpipe (void)
 	       return ret_error;
        }
 
+       if (sigaction (SIGBUS, &new_act, &old_act)) {
+               PRINT_ERROR ("sigaction (SIGBUS, ...): %s (%d)\n", strerror(errno), errno);
+	       return ret_error;
+       }
+
        return ret_ok;
 }
 
@@ -65,7 +70,6 @@ thread_routine (void *data)
 	set_sigpipe();	
 
         while (thread->exit == 0) {
-                cherokee_fdpoll_watch (thread->fdpoll, srv->fdwatch_msecs);
                 cherokee_thread_step (thread);
         }
 
@@ -87,11 +91,10 @@ cherokee_thread_new  (cherokee_thread_t **thd, void *server, int real_thread)
 	
 	cherokee_fdpoll_new(&n->fdpoll);
 
+	n->exit            = 0;
 	n->active_list_num = 0;
-	n->server = server;
-	n->exit = 0;
-
-	CHEROKEE_MUTEX_INIT (&n->active_list_lock, NULL);
+	n->server          = server;
+	n->real_thread     = real_thread;
 
 	/* Return the object
 	 */
@@ -122,24 +125,16 @@ conn_set_mode (cherokee_thread_t *thd, cherokee_connection_t *conn, cherokee_soc
 static void
 add_connection (cherokee_thread_t *thd, cherokee_connection_t *conn)
 {
-	CHEROKEE_MUTEX_LOCK (&thd->active_list_lock);
-
 	list_add ((list_t *)conn, &thd->active_list);
 	thd->active_list_num++;
-
-	CHEROKEE_MUTEX_UNLOCK (&thd->active_list_lock);
 }
 
 
 static void
 del_connection (cherokee_thread_t *thd, cherokee_connection_t *conn)
 {
-	CHEROKEE_MUTEX_LOCK (&thd->active_list_lock);
-
 	list_del ((list_t *)conn);
 	thd->active_list_num--;
-
-	CHEROKEE_MUTEX_UNLOCK (&thd->active_list_lock);
 }
 
 
@@ -150,19 +145,21 @@ purge_closed_connection (cherokee_thread_t *thread, cherokee_connection_t *conn)
 	 */
 	cherokee_fdpoll_del (thread->fdpoll, SOCKET_FD(conn->socket));
 
+	/* Delete from active connections list
+	 */
+	del_connection (thread, conn);
+
 	/* Maybe have a delayed log
 	 */
 	cherokee_connection_log_delayed (conn);
 
-	/* Clean the connection
+	/* Clean the connection and close the socket
 	 */
 	cherokee_connection_mrproper (conn);
 
-	/* Delete from active connections list and 
-	 * add it to the rusable one
-	 */
-	del_connection (thread, conn);
-	list_add ((list_t *)conn, &thread->reuse_list);
+	/* Add it to the rusable list
+	 */	
+	cherokee_thread_set_reusable_connection (thread, conn);
 }
 
 
@@ -232,11 +229,12 @@ process_active_connections (cherokee_thread_t *thd)
 		 */
 		conn->timeout = srv->bogo_now + srv->timeout;
 
+		
 		/* Phases
 		 */
 		switch (conn->phase) {
 		case phase_tls_handshake:
-			ret = cherokee_socket_init_tls (CONN_SOCK(conn), conn);
+			ret = cherokee_socket_init_tls (CONN_SOCK(conn), CONN_VSRV(conn));
 			switch (ret) {
 			case ret_eagain:
 				continue;
@@ -289,11 +287,18 @@ process_active_connections (cherokee_thread_t *thd)
 
 		case phase_reading_header:
 			ret = cherokee_connection_recv (conn, conn->buffer);
-			if (ret != ret_ok) {
+			switch (ret) {
+			case ret_ok:
+				break;
+			case ret_eagain:
+				continue;
+			case ret_eof:
 				purge_closed_connection (thd, conn);
+				continue;
+			default:
+				SHOULDNT_HAPPEN;
 			}
 
-
 			/* Check security
 			 */
 			ret = cherokee_connection_reading_check (conn);
@@ -382,6 +387,14 @@ process_active_connections (cherokee_thread_t *thd)
 				continue;
 			}
 
+			/* Check for IP validation
+			 */
+			ret = cherokee_connection_check_ip_validation (conn, plugin_entry);
+			if (ret != ret_ok) {
+				conn->phase = phase_add_headers;
+				continue;
+			}
+
 			/* Check for authentication
 			 */
 			ret = cherokee_connection_check_authentication (conn, plugin_entry);
@@ -439,6 +452,12 @@ process_active_connections (cherokee_thread_t *thd)
 					conn->error_code = http_moved_permanently;
 				}
 			}
+			
+			/* If it has mmaped content, go to next stage
+			 */
+			if (conn->mmaped != NULL) {
+				goto phase_add_headers_EXIT;
+			}
 
 			ret = cherokee_connection_send_header (conn);
 
@@ -485,11 +504,32 @@ process_active_connections (cherokee_thread_t *thd)
 			/* Maybe log the connection
 			 */
 			cherokee_connection_log_or_delay (conn);			
+
+		phase_add_headers_EXIT:
 			conn->phase = phase_steping;
 			
+
 		case phase_steping: {
 			ret_t sent_ret;
 			
+			/* Special case: 
+			 * If the content is mmap()ed, it has to send the header +
+			 * the file content and stop processing the connection.
+			 */
+			if (conn->mmaped != NULL) {
+				sent_ret = cherokee_connection_send_header_and_mmaped (conn);
+				switch (sent_ret) {
+				case ret_eagain:
+					continue;
+				case ret_error:
+					purge_closed_connection (thd, conn);
+					continue;
+				default:
+					maybe_purge_closed_connection (thd, conn);
+					continue;
+				}
+			}
+
 			/* Handler step: read or make new data to send
 			 */
 			ret = cherokee_connection_step (conn);
@@ -558,33 +598,118 @@ cherokee_thread_free (cherokee_thread_t *thd)
 
 	/* Free the connection
 	 */
-	list_for_each_safe (i, tmp, (list_t*)&thd->active_list) {
+	list_for_each_safe (i, tmp, &thd->active_list) {
 		cherokee_connection_free (CONN(i));
 	}
 
-	CHEROKEE_MUTEX_DESTROY (&thd->active_list_lock);
-
 	free (thd);
 
 	return ret_ok;
 }
 
 
-ret_t 
-cherokee_thread_step (cherokee_thread_t *thread)
+static int
+__accept_from_server (cherokee_thread_t *thd, int srv_socket, int is_tls)
 {
-	ret_t              ret = ret_ok;
-	cherokee_server_t *srv = THREAD_SRV(thread);
+	ret_t                  ret;
+	cherokee_connection_t *new_conn;
 
+        /* Return if there're no new connections
+         */
+        if (cherokee_fdpoll_check (thd->fdpoll, srv_socket, 0) == 0) {
+                return 0;
+        }
 
-	/* Process connections
+        /* Get a new connection object
+         */
+	ret = cherokee_thread_get_new_connection (thd, &new_conn);
+	if (ret < ret_ok) return 0;
+	
+	/* Accept the connection
 	 */
-	if (thread->active_list_num > 0) 
+	ret = cherokee_socket_accept (CONN_SOCK(new_conn), srv_socket);
+	if (ret < ret_ok) goto error;
+	
+	/* Set the connection time-out
+	 */
+	new_conn->timeout = THREAD_SRV(thd)->bogo_now + THREAD_SRV(thd)->timeout;
+
+	/* May active the TLS support
+         */
+        if (is_tls) {
+                new_conn->phase = phase_tls_handshake;
+        }
+	
+	/* Add the new connection to the thread
+	 */
+	cherokee_thread_add_connection (thd, new_conn);
+	return 1;
+
+error:
+	cherokee_thread_set_reusable_connection (thd, new_conn);
+	return 0;
+}
+
+
+ret_t 
+cherokee_thread_step_SINGLE_THREAD (cherokee_thread_t *thd)
+{
+	cherokee_server_t *srv = THREAD_SRV(thd);
+	ret_t ret;
+	int re;
+
+	cherokee_fdpoll_reset (thd->fdpoll, srv->socket);
+
+	re = cherokee_fdpoll_watch (thd->fdpoll, srv->fdwatch_msecs);
+	if (re <= 0) return ret_ok;
+
+	re = __accept_from_server (thd, srv->socket, 0);		
+
+	return process_active_connections (thd);
+}
+
+
+ret_t 
+cherokee_thread_step_MULTI_THREAD (cherokee_thread_t *thd)
+{
+	int i = MAX_NEW_CONNECTIONS_PER_STEP; 
+	int re;
+	ret_t ret;
+	cherokee_server_t *srv = THREAD_SRV(thd);
+
+	if (thd->active_list_num == 0) {
+		CHEROKEE_MUTEX_LOCK (&THREAD_SRV(thd)->accept_mutex);
+
+		cherokee_fdpoll_add   (thd->fdpoll, srv->socket, 0);
+		cherokee_fdpoll_reset (thd->fdpoll, srv->socket);
+		cherokee_fdpoll_watch (thd->fdpoll, srv->fdwatch_msecs);
+
+		re = __accept_from_server (thd, srv->socket, 0);		
+
+		cherokee_fdpoll_del (thd->fdpoll, srv->socket);
+
+		CHEROKEE_MUTEX_UNLOCK (&THREAD_SRV(thd)->accept_mutex);
+	} else 
 	{
-		ret = process_active_connections (thread);
+		int unlocked;
+		
+		unlocked = CHEROKEE_MUTEX_TRY_LOCK (&THREAD_SRV(thd)->accept_mutex);
+		if (! unlocked) {
+			cherokee_fdpoll_add   (thd->fdpoll, srv->socket, 0);
+			cherokee_fdpoll_reset (thd->fdpoll, srv->socket);
+			cherokee_fdpoll_watch (thd->fdpoll, srv->fdwatch_msecs);
+
+			re = __accept_from_server (thd, srv->socket, 0);		
+
+			cherokee_fdpoll_del (thd->fdpoll, srv->socket);
+
+			CHEROKEE_MUTEX_UNLOCK (&THREAD_SRV(thd)->accept_mutex);
+		} else {
+			cherokee_fdpoll_watch (thd->fdpoll, srv->fdwatch_msecs);
+		}
 	}
 
-	return ret;
+	return process_active_connections (thd);
 }
 
 
@@ -597,22 +722,25 @@ cherokee_thread_get_new_connection (cherokee_thread_t *thd, cherokee_connection_
 	server = thd->server;
 
 	if (list_empty (&thd->reuse_list)) {
+		ret_t ret;
+
 		/* Create new connection object
 		 */
-		cherokee_connection_new (&new_connection);
+		ret = cherokee_connection_new (&new_connection);
+		if (ret < ret_ok) return ret;
 	} else {
 		/* Reuse an old one
 		 */
-		new_connection = CONN(thd->reuse_list.next);
-		list_del ((list_t *)new_connection);	
-		new_connection->timeout = server->bogo_now + server->timeout;
+		new_connection = CONN(thd->reuse_list.prev);
+		list_del ((list_t *)new_connection);
+		INIT_LIST_HEAD((list_t *)new_connection);
 	}
 
 	/* Set the basic information to the connection
 	 */
 	CONN_SRV(new_connection)  = server;
 	CONN_VSRV(new_connection) = server->vserver_default;
-
+		
 	*conn = new_connection;
 
 	return ret_ok;
@@ -631,14 +759,12 @@ cherokee_thread_add_connection (cherokee_thread_t *thd, cherokee_connection_t  *
 {
 	cherokee_fdpoll_add (thd->fdpoll, SOCKET_FD(conn->socket), 0);
 	conn_set_mode (thd, conn, socket_reading);
-
 	add_connection (thd, conn);
 
 	return ret_ok;
 }
 
 
-
 int   
 cherokee_thread_connection_num (cherokee_thread_t *thd) 
 {
@@ -650,7 +776,7 @@ ret_t
 cherokee_thread_close_all_connections (cherokee_thread_t *thd)
 {
 	list_t *i, *tmp;
-	list_for_each_safe (i, tmp, (list_t*)&thd->active_list) {
+	list_for_each_safe (i, tmp, &thd->active_list) {
 		purge_closed_connection (thd, CONN(i));
 	}
 
